@@ -3,7 +3,7 @@ A history-aware add command to help with data entry.
 |-}
 
 {-# OPTIONS_GHC -fno-warn-missing-signatures -fno-warn-unused-do-bind #-}
-{-# LANGUAGE CPP, ScopedTypeVariables, DeriveDataTypeable, RecordWildCards, TypeOperators, FlexibleContexts #-}
+{-# LANGUAGE CPP, ScopedTypeVariables, DeriveDataTypeable, RecordWildCards, TypeOperators, FlexibleContexts, OverloadedStrings #-}
 
 module Hledger.Cli.Add
 where
@@ -12,11 +12,15 @@ import Prelude ()
 import Prelude.Compat
 import Control.Exception as E
 import Control.Monad
+import Control.Monad.Trans.Class
+import Control.Monad.State.Strict (evalState, evalStateT)
 import Control.Monad.Trans (liftIO)
 import Data.Char (toUpper, toLower)
 import Data.List.Compat
 import qualified Data.Set as S
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T
 import Data.Time.Calendar (Day)
 import Data.Typeable (Typeable)
 import Safe (headDef, headMay)
@@ -26,7 +30,8 @@ import System.Console.Haskeline.Completion
 import System.Console.Wizard
 import System.Console.Wizard.Haskeline
 import System.IO ( stderr, hPutStr, hPutStrLn )
-import Text.Parsec
+import Text.Megaparsec
+import Text.Megaparsec.Text
 import Text.Printf
 
 import Hledger
@@ -84,7 +89,7 @@ add opts j
         showHelp
         today <- getCurrentDay
         let es = defEntryState{esOpts=opts
-                              ,esArgs=map stripquotes $ listofstringopt "args" $ rawopts_ opts
+                              ,esArgs=map (T.unpack . stripquotes . T.pack) $ listofstringopt "args" $ rawopts_ opts
                               ,esToday=today
                               ,esDefDate=today
                               ,esJournal=j
@@ -160,7 +165,7 @@ transactionWizard es@EntryState{..} = do
   balancedPostingsWizard
 
 -- Identify the closest recent match for this description in past transactions.
-similarTransaction :: EntryState -> String -> Maybe Transaction
+similarTransaction :: EntryState -> Text -> Maybe Transaction
 similarTransaction EntryState{..} desc =
   let q = queryFromOptsOnly esToday $ reportopts_ esOpts
       historymatches = transactionsSimilarTo esJournal q desc
@@ -181,14 +186,14 @@ dateAndCodeWizard EntryState{..} = do
     where
       parseSmartDateAndCode refdate s = either (const Nothing) (\(d,c) -> return (fixSmartDate refdate d, c)) edc
           where
-            edc = runParser (dateandcodep <* eof) nullctx "" $ lowercase s
-            dateandcodep :: Stream [Char] m t => ParsecT [Char] JournalContext m (SmartDate, String)
+            edc = runParser (dateandcodep <* eof) "" $ T.pack $ lowercase s
+            dateandcodep :: Parser (SmartDate, Text)
             dateandcodep = do
                 d <- smartdate
-                c <- optionMaybe codep
+                c <- optional codep
                 many spacenonewline
                 eof
-                return (d, fromMaybe "" c)
+                return (d, T.pack $ fromMaybe "" c)
       -- defday = fixSmartDate today $ fromparse $ (parse smartdate "" . lowercase) defdate
       -- datestr = showDate $ fixSmartDate defday smtdate
 
@@ -198,8 +203,8 @@ descriptionAndCommentWizard EntryState{..} = do
        defaultTo' def $ nonEmpty $
        maybeRestartTransaction $
        line $ green $ printf "Description%s: " (showDefault def)
-  let (desc,comment) = (strip a, strip $ dropWhile (==';') b) where (a,b) = break (==';') s
-  return (desc,comment)
+  let (desc,comment) = (T.pack $ strip a, T.pack $ strip $ dropWhile (==';') b) where (a,b) = break (==';') s
+  return (desc, comment)
 
 postingsWizard es@EntryState{..} = do
   mp <- postingWizard es
@@ -216,10 +221,10 @@ postingWizard es@EntryState{..} = do
   else do
     let es1 = es{esArgs=drop 1 esArgs}
     (amt,comment)  <- amountAndCommentWizard es1
-    return $ Just nullposting{paccount=stripbrackets acct
+    return $ Just nullposting{paccount=T.pack $ stripbrackets acct
                              ,pamount=Mixed [amt]
                              ,pcomment=comment
-                             ,ptype=accountNamePostingType acct
+                             ,ptype=accountNamePostingType $ T.pack acct
                              }
 
 postingsBalanced :: [Posting] -> Bool
@@ -242,13 +247,18 @@ accountWizard EntryState{..} = do
    line $ green $ printf "Account %d%s%s: " pnum (endmsg::String) (showDefault def)
     where
       canfinish = not (null esPostings) && postingsBalanced esPostings
+      parseAccountOrDotOrNull :: String -> Bool -> String -> Maybe String
       parseAccountOrDotOrNull _  _ "."       = dbg1 $ Just "." -- . always signals end of txn
       parseAccountOrDotOrNull "" True ""     = dbg1 $ Just ""  -- when there's no default and txn is balanced, "" also signals end of txn
       parseAccountOrDotOrNull def@(_:_) _ "" = dbg1 $ Just def -- when there's a default, "" means use that
-      parseAccountOrDotOrNull _ _ s          = dbg1 $ either (const Nothing) validateAccount $ runParser (accountnamep <* eof) (jContext esJournal) "" s -- otherwise, try to parse the input as an accountname
+      parseAccountOrDotOrNull _ _ s          = dbg1 $ fmap T.unpack $
+        either (const Nothing) validateAccount $
+          flip evalState esJournal $ runParserT (accountnamep <* eof) "" (T.pack s) -- otherwise, try to parse the input as an accountname
+        where
+          validateAccount :: Text -> Maybe Text
+          validateAccount t | no_new_accounts_ esOpts && not (t `elem` journalAccountNames esJournal) = Nothing
+                            | otherwise = Just t
       dbg1 = id -- strace
-      validateAccount s | no_new_accounts_ esOpts && not (s `elem` journalAccountNames esJournal) = Nothing
-                        | otherwise = Just s
 
 amountAndCommentWizard EntryState{..} = do
   let pnum = length esPostings + 1
@@ -260,7 +270,7 @@ amountAndCommentWizard EntryState{..} = do
       def = case (esArgs, mhistoricalp, followedhistoricalsofar) of
               (d:_,_,_)                                             -> d
               (_,Just hp,True)                                      -> showamt $ pamount hp
-              _  | pnum > 1 && not (isZeroMixedAmount balancingamt) -> showamt balancingamt
+              _  | pnum > 1 && not (isZeroMixedAmount balancingamt) -> showamt balancingamtfirstcommodity
               _                                                     -> ""
   retryMsg "A valid hledger amount is required. Eg: 1, $2, 3 EUR, \"4 red apples\"." $
    parser parseAmountAndComment $
@@ -269,17 +279,23 @@ amountAndCommentWizard EntryState{..} = do
    maybeRestartTransaction $
    line $ green $ printf "Amount  %d%s: " pnum (showDefault def)
     where
-      parseAmountAndComment = either (const Nothing) Just . runParser (amountandcommentp <* eof) nodefcommodityctx ""
-      nodefcommodityctx = (jContext esJournal){ctxDefaultCommodityAndStyle=Nothing}
-      amountandcommentp :: Stream [Char] m t => ParsecT [Char] JournalContext m (Amount, String)
+      parseAmountAndComment s = either (const Nothing) Just $
+                                runParser
+                                  (evalStateT (amountandcommentp <* eof) nodefcommodityj)
+                                  ""
+                                  (T.pack s)
+      nodefcommodityj = esJournal{jparsedefaultcommodity=Nothing}
+      amountandcommentp :: JournalParser (Amount, Text)
       amountandcommentp = do
         a <- amountp
-        many spacenonewline
-        c <- fromMaybe "" `fmap` optionMaybe (char ';' >> many anyChar)
+        lift (many spacenonewline)
+        c <- T.pack <$> fromMaybe "" `fmap` optional (char ';' >> many anyChar)
         -- eof
         return (a,c)
       balancingamt = negate $ sum $ map pamount realps where realps = filter isReal esPostings
-      showamt = showMixedAmountWithPrecision
+      balancingamtfirstcommodity = Mixed $ take 1 $ amounts balancingamt
+      showamt =
+        showMixedAmountWithPrecision
                   -- what should this be ?
                   -- 1 maxprecision (show all decimal places or none) ?
                   -- 2 maxprecisionwithpoint (show all decimal places or .0 - avoids some but not all confusion with thousands separators) ?
@@ -290,11 +306,11 @@ amountAndCommentWizard EntryState{..} = do
                   maxprecisionwithpoint
   --
   -- let -- (amt,comment) = (strip a, strip $ dropWhile (==';') b) where (a,b) = break (==';') amtcmt
-      -- a           = fromparse $ runParser (amountp <|> return missingamt) (jContext esJournal) "" amt
-  --     awithoutctx = fromparse $ runParser (amountp <|> return missingamt) nullctx              "" amt
+      -- a           = fromparse $ runParser (amountp <|> return missingamt) (jparsestate esJournal) "" amt
+  --     awithoutjps = fromparse $ runParser (amountp <|> return missingamt) mempty              "" amt
   --     defamtaccepted = Just (showAmount a) == mdefamt
   --     es2 = if defamtaccepted then es1 else es1{esHistoricalPostings=Nothing}
-  --     mdefaultcommodityapplied = if acommodity a == acommodity awithoutctx then Nothing else Just $ acommodity a
+  --     mdefaultcommodityapplied = if acommodity a == acommodity awithoutjps then Nothing else Just $ acommodity a
   -- when (isJust mdefaultcommodityapplied) $
   --      liftIO $ hPutStrLn stderr $ printf "using default commodity (%s)" (fromJust mdefaultcommodityapplied)
 
@@ -312,10 +328,10 @@ dateCompleter :: String -> CompletionFunc IO
 dateCompleter = completer ["today","tomorrow","yesterday"]
 
 descriptionCompleter :: Journal -> String -> CompletionFunc IO
-descriptionCompleter j = completer (journalDescriptions j)
+descriptionCompleter j = completer (map T.unpack $ journalDescriptions j)
 
 accountCompleter :: Journal -> String -> CompletionFunc IO
-accountCompleter j = completer (journalAccountNamesUsed j)
+accountCompleter j = completer (map T.unpack $ journalAccountNamesUsed j)
 
 amountCompleter :: String -> CompletionFunc IO
 amountCompleter = completer []
@@ -376,7 +392,7 @@ ensureOneNewlineTerminated = (++"\n") . reverse . dropWhile (=='\n') . reverse
 registerFromString :: String -> IO String
 registerFromString s = do
   d <- getCurrentDay
-  j <- readJournal' s
+  j <- readJournal' $ T.pack s
   return $ postingsReportAsText opts $ postingsReport ropts (queryFromOpts d ropts) j
       where
         ropts = defreportopts{empty_=True}
@@ -389,7 +405,7 @@ capitalize (c:cs) = toUpper c : cs
 -- | Find the most similar and recent transactions matching the given
 -- transaction description and report query.  Transactions are listed
 -- with their "relevancy" score, most relevant first.
-transactionsSimilarTo :: Journal -> Query -> String -> [(Double,Transaction)]
+transactionsSimilarTo :: Journal -> Query -> Text -> [(Double,Transaction)]
 transactionsSimilarTo j q desc =
     sortBy compareRelevanceAndRecency
                $ filter ((> threshold).fst)
@@ -403,11 +419,11 @@ transactionsSimilarTo j q desc =
 -- descriptions.  This is like compareStrings, but first strips out
 -- any numbers, to improve accuracy eg when there are bank transaction
 -- ids from imported data.
-compareDescriptions :: String -> String -> Double
+compareDescriptions :: Text -> Text -> Double
 compareDescriptions s t = compareStrings s' t'
-    where s' = simplify s
-          t' = simplify t
-          simplify = filter (not . (`elem` "0123456789"))
+    where s' = simplify $ T.unpack s
+          t' = simplify $ T.unpack t
+          simplify = filter (not . (`elem` ("0123456789" :: String)))
 
 -- | Return a similarity measure, from 0 to 1, for two strings.  This
 -- was based on Simon White's string similarity algorithm
