@@ -9,7 +9,7 @@ Here is a description/specification for the balance command.  See also
 
 /Basic balance report/
 
-With no reporting interval (@--monthly@ etc.), hledger's balance
+With no report interval (@--monthly@ etc.), hledger's balance
 command emulates ledger's, showing accounts indented according to
 hierarchy, along with their total amount posted (including subaccounts).
 
@@ -232,19 +232,24 @@ Currently, empty cells show 0.
 
 -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Hledger.Cli.Balance (
   balancemode
  ,balance
  ,balanceReportAsText
  ,balanceReportItemAsText
- ,periodBalanceReportAsText
- ,cumulativeBalanceReportAsText
+ ,periodChangeReportAsText
+ ,cumulativeChangeReportAsText
  ,historicalBalanceReportAsText
  ,tests_Hledger_Cli_Balance
 ) where
 
 import Data.List (intercalate)
 import Data.Maybe (fromMaybe, isJust)
+import Data.Monoid
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import System.Console.CmdArgs.Explicit as C
 import Text.CSV
 import Test.HUnit
@@ -262,17 +267,21 @@ balancemode = (defCommandMode $ ["balance"] ++ aliases) { -- also accept but don
   modeHelp = "show accounts and balances" `withAliases` aliases
  ,modeGroupFlags = C.Group {
      groupUnnamed = [
-      flagNone ["tree"] (\opts -> setboolopt "tree" opts) "show accounts as a tree (default in simple reports)"
-     ,flagNone ["flat"] (\opts -> setboolopt "flat" opts) "show accounts as a list (default in multicolumn mode)"
-     ,flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "flat mode: omit N leading account name parts"
-     ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "singlecolumn mode: use this custom line format"
-     ,flagNone ["no-elide"] (\opts -> setboolopt "no-elide" opts) "tree mode: don't squash boring parent accounts"
-     ,flagNone ["historical","H"] (\opts -> setboolopt "historical" opts) "multicolumn mode: show historical ending balances"
-     ,flagNone ["cumulative"] (\opts -> setboolopt "cumulative" opts) "multicolumn mode: show accumulated ending balances"
-     ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "multicolumn mode: show a row average column"
-     ,flagNone ["row-total","T"] (\opts -> setboolopt "row-total" opts) "multicolumn mode: show a row total column"
-     ,flagNone ["no-total","N"] (\opts -> setboolopt "no-total" opts) "don't show the final total row"
-     ,flagNone ["value","V"] (setboolopt "value") "show amounts as their current market value in their default valuation commodity"
+      flagNone ["change"] (\opts -> setboolopt "change" opts)
+        "show balance change in each period (default)"
+     ,flagNone ["cumulative"] (\opts -> setboolopt "cumulative" opts)
+        "show balance change accumulated across periods (in multicolumn reports)"
+     ,flagNone ["historical","H"] (\opts -> setboolopt "historical" opts)
+        "show historical ending balance in each period (includes postings before report start date)\n "
+     ,flagNone ["tree"] (\opts -> setboolopt "tree" opts) "show accounts as a tree; amounts include subaccounts (default in simple reports)"
+     ,flagNone ["flat"] (\opts -> setboolopt "flat" opts) "show accounts as a list; amounts exclude subaccounts except when account is depth-clipped (default in multicolumn reports)\n "
+     ,flagNone ["value","V"] (setboolopt "value") "convert amounts to current market value in their default valuation commodity"
+     ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "show a row average column (in multicolumn reports)"
+     ,flagNone ["row-total","T"] (\opts -> setboolopt "row-total" opts) "show a row total column (in multicolumn reports)"
+     ,flagNone ["no-total","N"] (\opts -> setboolopt "no-total" opts) "omit the final total row"
+     ,flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "omit N leading account name parts (in flat mode)"
+     ,flagNone ["no-elide"] (\opts -> setboolopt "no-elide" opts) "don't squash boring parent accounts (in tree mode)"
+     ,flagReq  ["format"] (\s opts -> Right $ setopt "format" s opts) "FORMATSTR" "use this custom line format (in simple reports)"
      ]
      ++ outputflags
     ,groupHidden = []
@@ -289,12 +298,22 @@ balance opts@CliOpts{reportopts_=ropts} j = do
     Left err -> error' $ unlines [err]
     Right _ -> do
       let format   = outputFormatFromOpts opts
-          interval = intervalFromOpts ropts
+          interval = interval_ ropts
           baltype  = balancetype_ ropts
           valuedate = fromMaybe d $ queryEndDate False $ queryFromOpts d ropts
+      -- shenanigans: use single/multiBalanceReport when we must,
+      -- ie when there's a report interval, or --historical or -- cumulative.
+      -- Otherwise prefer the older balanceReport since it can elide boring parents.
       case interval of
         NoInterval -> do
-          let report = balanceReport ropts (queryFromOpts d ropts) j
+          let report
+                -- For --historical/--cumulative, we must use multiBalanceReport.
+                -- (This forces --no-elide.)
+                | balancetype_ ropts `elem` [HistoricalBalance, CumulativeChange]
+                  = let ropts' | flat_ ropts = ropts
+                               | otherwise   = ropts{accountlistmode_=ALTree}
+                    in singleBalanceReport ropts' (queryFromOpts d ropts) j
+                | otherwise = balanceReport ropts (queryFromOpts d ropts) j
               convert | value_ ropts = balanceReportValue j valuedate
                       | otherwise    = id
               render = case format of
@@ -308,8 +327,8 @@ balance opts@CliOpts{reportopts_=ropts} j = do
               render = case format of
                 "csv" -> \ropts r -> (++ "\n") $ printCSV $ multiBalanceReportAsCsv ropts r
                 _     -> case baltype of
-                  PeriodBalance     -> periodBalanceReportAsText
-                  CumulativeBalance -> cumulativeBalanceReportAsText
+                  PeriodChange     -> periodChangeReportAsText
+                  CumulativeChange -> cumulativeChangeReportAsText
                   HistoricalBalance -> historicalBalanceReportAsText
           writeOutput opts $ render ropts $ convert report
 
@@ -320,14 +339,14 @@ balance opts@CliOpts{reportopts_=ropts} j = do
 -- in which it has most recently been market-priced, ie the commodity
 -- mentioned in the most recent applicable historical price directive
 -- before this date.
--- defaultValuationCommodity :: Journal -> Day -> Commodity -> Maybe Commodity
+-- defaultValuationCommodity :: Journal -> Day -> CommoditySymbol -> Maybe CommoditySymbol
 -- defaultValuationCommodity j d c = mpamount <$> commodityValue j d c
 
 -- | Render a single-column balance report as CSV.
 balanceReportAsCsv :: ReportOpts -> BalanceReport -> CSV
 balanceReportAsCsv opts (items, total) =
   ["account","balance"] :
-  [[a, showMixedAmountWithoutPrice b] | ((a, _, _), b) <- items]
+  [[T.unpack a, showMixedAmountOneLineWithoutPrice b] | (a, _, _, b) <- items]
   ++
   if no_total_ opts
   then []
@@ -348,8 +367,8 @@ balanceReportAsText opts ((items, total)) = unlines $ concat lines ++ t
                Right fmt ->
                 let
                   -- abuse renderBalanceReportItem to render the total with similar format
-                  acctcolwidth = maximum' [length fullname | ((fullname, _, _), _) <- items]
-                  totallines = map rstrip $ renderBalanceReportItem fmt (replicate (acctcolwidth+1) ' ', 0, total)
+                  acctcolwidth = maximum' [T.length fullname | (fullname, _, _, _) <- items]
+                  totallines = map rstrip $ renderBalanceReportItem fmt (T.replicate (acctcolwidth+1) " ", 0, total)
                   -- with a custom format, extend the line to the full report width;
                   -- otherwise show the usual 20-char line for compatibility
                   overlinewidth | isJust (format_ opts) = maximum' $ map length $ concat lines
@@ -374,6 +393,7 @@ tests_balanceReportAsText = [
  ]
 
 {-
+:r
 This implementation turned out to be a bit convoluted but implements the following algorithm for formatting:
 
 - If there is a single amount, print it with the account name directly:
@@ -388,7 +408,7 @@ This implementation turned out to be a bit convoluted but implements the followi
 -- differently-priced quantities of the same commodity will appear merged.
 -- The output will be one or more lines depending on the format and number of commodities.
 balanceReportItemAsText :: ReportOpts -> StringFormat -> BalanceReportItem -> [String]
-balanceReportItemAsText opts fmt ((_, accountName, depth), amt) =
+balanceReportItemAsText opts fmt (_, accountName, depth, amt) =
   renderBalanceReportItem fmt (
     maybeAccountNameDrop opts accountName,
     depth,
@@ -417,7 +437,7 @@ renderComponent (acctname, depth, total) (FormatField ljust min max field) = cas
                       where d = case min of
                                  Just m  -> depth * m
                                  Nothing -> depth
-  AccountField     -> formatString ljust min max acctname
+  AccountField     -> formatString ljust min max (T.unpack acctname)
   TotalField       -> fitStringMulti min max True False $ showMixedAmountWithoutPrice total
   _                -> ""
 
@@ -428,7 +448,7 @@ renderComponent (acctname, depth, total) (FormatField ljust min max field) = cas
 renderComponent1 :: (AccountName, Int, MixedAmount) -> StringFormatComponent -> String
 renderComponent1 _ (FormatLiteral s) = s
 renderComponent1 (acctname, depth, total) (FormatField ljust min max field) = case field of
-  AccountField     -> formatString ljust min max ((intercalate ", " . lines) (indented acctname))
+  AccountField     -> formatString ljust min max ((intercalate ", " . lines) (indented (T.unpack acctname)))
                       where
                         -- better to indent the account name here rather than use a DepthField component
                         -- so that it complies with width spec. Uses a fixed indent step size.
@@ -445,12 +465,12 @@ multiBalanceReportAsCsv opts (MultiBalanceReport (colspans, items, (coltotals,to
    ++ (if row_total_ opts then ["total"] else [])
    ++ (if average_ opts then ["average"] else [])
   ) :
-  [a : a' : show i :
+  [T.unpack a : T.unpack a' : show i :
    map showMixedAmountOneLineWithoutPrice
    (amts
     ++ (if row_total_ opts then [rowtot] else [])
     ++ (if average_ opts then [rowavg] else []))
-  | ((a,a',i), amts, rowtot, rowavg) <- items]
+  | (a,a',i, amts, rowtot, rowavg) <- items]
   ++
   if no_total_ opts
   then []
@@ -462,15 +482,15 @@ multiBalanceReportAsCsv opts (MultiBalanceReport (colspans, items, (coltotals,to
            )]
 
 -- | Render a multi-column period balance report as plain text suitable for console output.
-periodBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
-periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+periodChangeReportAsText :: ReportOpts -> MultiBalanceReport -> String
+periodChangeReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
   unlines $
   ([printf "Balance changes in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
   trimborder $ lines $
    render id (" "++) showMixedAmountOneLineWithoutPrice $
     addtotalrow $
      Table
-     (T.Group NoLine $ map (Header . padRightWide acctswidth) accts)
+     (T.Group NoLine $ map (Header . padRightWide acctswidth . T.unpack) accts)
      (T.Group NoLine $ map Header colheadings)
      (map rowvals items')
   where
@@ -481,11 +501,11 @@ periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotal
     items' | empty_ opts = items
            | otherwise   = items -- dbg1 "2" $ filter (any (not . isZeroMixedAmount) . snd) $ dbg1 "1" items
     accts = map renderacct items'
-    renderacct ((a,a',i),_,_,_)
-      | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
+    renderacct (a,a',i,_,_,_)
+      | tree_ opts = T.replicate ((i-1)*2) " " <> a'
       | otherwise  = maybeAccountNameDrop opts a
-    acctswidth = maximum' $ map strWidth accts
-    rowvals (_,as,rowtot,rowavg) = as
+    acctswidth = maximum' $ map textWidth accts
+    rowvals (_,_,_,as,rowtot,rowavg) = as
                                    ++ (if row_total_ opts then [rowtot] else [])
                                    ++ (if average_ opts then [rowavg] else [])
     addtotalrow | no_total_ opts = id
@@ -496,8 +516,8 @@ periodBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotal
                                     ))
 
 -- | Render a multi-column cumulative balance report as plain text suitable for console output.
-cumulativeBalanceReportAsText :: ReportOpts -> MultiBalanceReport -> String
-cumulativeBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
+cumulativeChangeReportAsText :: ReportOpts -> MultiBalanceReport -> String
+cumulativeChangeReportAsText opts r@(MultiBalanceReport (colspans, items, (coltotals,tot,avg))) =
   unlines $
   ([printf "Ending balances (cumulative) in %s:" (showDateSpan $ multiBalanceReportSpan r)] ++) $
   trimborder $ lines $
@@ -513,11 +533,11 @@ cumulativeBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
                   ++ (if row_total_ opts then ["  Total"] else [])
                   ++ (if average_ opts then ["Average"] else [])
     accts = map renderacct items
-    renderacct ((a,a',i),_,_,_)
-      | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
-      | otherwise  = maybeAccountNameDrop opts a
+    renderacct (a,a',i,_,_,_)
+      | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
+      | otherwise  = T.unpack $ maybeAccountNameDrop opts a
     acctswidth = maximum' $ map strWidth accts
-    rowvals (_,as,rowtot,rowavg) = as
+    rowvals (_,_,_,as,rowtot,rowavg) = as
                                    ++ (if row_total_ opts then [rowtot] else [])
                                    ++ (if average_ opts then [rowavg] else [])
     addtotalrow | no_total_ opts = id
@@ -545,11 +565,11 @@ historicalBalanceReportAsText opts r@(MultiBalanceReport (colspans, items, (colt
                   ++ (if row_total_ opts then ["  Total"] else [])
                   ++ (if average_ opts then ["Average"] else [])
     accts = map renderacct items
-    renderacct ((a,a',i),_,_,_)
-      | tree_ opts = replicate ((i-1)*2) ' ' ++ a'
-      | otherwise  = maybeAccountNameDrop opts a
+    renderacct (a,a',i,_,_,_)
+      | tree_ opts = replicate ((i-1)*2) ' ' ++ T.unpack a'
+      | otherwise  = T.unpack $ maybeAccountNameDrop opts a
     acctswidth = maximum' $ map strWidth accts
-    rowvals (_,as,rowtot,rowavg) = as
+    rowvals (_,_,_,as,rowtot,rowavg) = as
                              ++ (if row_total_ opts then [rowtot] else [])
                              ++ (if average_ opts then [rowavg] else [])
     addtotalrow | no_total_ opts = id
