@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP, ScopedTypeVariables, DeriveDataTypeable, FlexibleContexts #-}
 {-|
 
 Common cmdargs modes and flags, a command-line options type, and
@@ -6,6 +5,8 @@ related utilities used by hledger commands.
 
 -}
 
+{-# LANGUAGE CPP, ScopedTypeVariables, DeriveDataTypeable, FlexibleContexts, TypeFamilies #-}
+
 module Hledger.Cli.CliOptions (
 
   -- * cmdargs flags & modes
@@ -21,7 +22,7 @@ module Hledger.Cli.CliOptions (
   defCommandMode,
   defAddonCommandMode,
   argsFlag,
-  showModeHelp,
+  showModeUsage,
   withAliases,
 
   -- * CLI options
@@ -53,6 +54,7 @@ module Hledger.Cli.CliOptions (
 
   -- * Other utils
   hledgerAddons,
+  topicForMode,
 
   -- * Tests
   tests_Hledger_Cli_CliOptions
@@ -64,14 +66,18 @@ import Prelude ()
 import Prelude.Compat
 import qualified Control.Exception as C
 import Control.Monad (when)
+import Data.Default
 #if !MIN_VERSION_base(4,8,0)
 import Data.Functor.Compat ((<$>))
 #endif
+import Data.Functor.Identity (Identity)
 import Data.List.Compat
 import Data.List.Split (splitOneOf)
 import Data.Maybe
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import Safe
-import System.Console.CmdArgs
+import System.Console.CmdArgs hiding (Default,def)
 import System.Console.CmdArgs.Explicit
 import System.Console.CmdArgs.Text
 #ifndef mingw32_HOST_OS
@@ -82,9 +88,10 @@ import System.Environment
 import System.Exit (exitSuccess)
 import System.FilePath
 import Test.HUnit
-import Text.Parsec
+import Text.Megaparsec
 
 import Hledger
+import Hledger.Cli.DocFiles
 import Hledger.Cli.Version
 
 
@@ -93,9 +100,12 @@ import Hledger.Cli.Version
 -- | Common help flags: --help, --debug, --version...
 helpflags :: [Flag RawOpts]
 helpflags = [
-  flagNone ["help","h"] (setboolopt "help") "show general help or (after command) command help"
+  flagNone ["h"]    (setboolopt "h")    "show general usage or (after COMMAND, the command's usage"
+ ,flagNone ["help"] (setboolopt "help") "show the current program's manual as plain text (or after an add-on COMMAND, the add-on's manual)"
+ ,flagNone ["man"]  (setboolopt "man")  "show the current program's manual with man"
+ ,flagNone ["info"] (setboolopt "info") "show the current program's manual with info"
  -- ,flagNone ["browse-args"] (setboolopt "browse-args") "use a web UI to select options and build up a command line"
- ,flagReq  ["debug"]    (\s opts -> Right $ setopt "debug" s opts) "N" "show debug output if N is 1-9 (default: 0)"
+ ,flagReq  ["debug"]    (\s opts -> Right $ setopt "debug" s opts) "[N]" "show debug output (levels 1-9, default: 1)"
  ,flagNone ["version"] (setboolopt "version") "show version information"
  ]
 
@@ -109,7 +119,8 @@ inputflags = [
   flagReq ["file","f"]  (\s opts -> Right $ setopt "file" s opts) "FILE" "use a different input file. For stdin, use -"
  ,flagReq ["rules-file"]  (\s opts -> Right $ setopt "rules-file" s opts) "RFILE" "CSV conversion rules file (default: FILE.rules)"
  ,flagReq ["alias"]  (\s opts -> Right $ setopt "alias" s opts)  "OLD=NEW" "display accounts named OLD as NEW"
- ,flagNone ["ignore-assertions"] (setboolopt "ignore-assertions") "ignore any balance assertions in the journal"
+ ,flagNone ["ignore-assertions","I"] (setboolopt "ignore-assertions") "ignore any balance assertions in the journal"
+ ,flagReq ["pivot"]  (\s opts -> Right $ setopt "pivot" s opts)  "TAG" "Replace the accounts of postings with TAG by TAG:<value>"
  ]
 
 -- | Common report-related flags: --period, --cost, etc.
@@ -122,22 +133,23 @@ reportflags = [
  ,flagNone ["monthly","M"]   (setboolopt "monthly") "multiperiod/multicolumn report by month"
  ,flagNone ["quarterly","Q"] (setboolopt "quarterly") "multiperiod/multicolumn report by quarter"
  ,flagNone ["yearly","Y"]    (setboolopt "yearly") "multiperiod/multicolumn report by year"
- ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or reporting interval all at once (overrides the flags above)"
- ,flagNone ["date2","aux-date"] (setboolopt "date2") "use postings/txns' secondary dates instead"
+ ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or report interval all at once (overrides the flags above)"
+ ,flagNone ["date2"]         (setboolopt "date2") "show, and make -b/-e/-p/date: match, secondary dates instead"
 
  ,flagNone ["cleared","C"]   (setboolopt "cleared") "include only cleared postings/txns"
  ,flagNone ["pending"]       (setboolopt "pending") "include only pending postings/txns"
  ,flagNone ["uncleared","U"] (setboolopt "uncleared") "include only uncleared (and pending) postings/txns"
  ,flagNone ["real","R"]      (setboolopt "real") "include only non-virtual postings"
  ,flagReq  ["depth"]         (\s opts -> Right $ setopt "depth" s opts) "N" "hide accounts/postings deeper than N"
- ,flagNone ["empty","E"]     (setboolopt "empty") "show empty/zero things which are normally omitted"
+ ,flagNone ["empty","E"]     (setboolopt "empty") "show items with zero amount, normally hidden"
  ,flagNone ["cost","B"]      (setboolopt "cost") "show amounts in their cost price's commodity"
+ ,flagNone ["anon"]              (setboolopt "anon") "output ledger with anonymized accounts and payees."
  ]
 
 -- | Common output-related flags: --output-file, --output-format...
 outputflags = [
-   flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE[.FMT]" "write output to FILE instead of stdout. A recognised FMT suffix influences the format."
-  ,flagReq  ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT" "select the output format. Supported formats: txt, csv."
+   flagReq  ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT" "select the output format. Supported formats:\ntxt, csv."
+  ,flagReq  ["output-file","o"]   (\s opts -> Right $ setopt "output-file" s opts) "FILE" "write output to FILE. A file extension matching one of the above formats selects that format."
   ]
 
 argsFlag :: FlagHelp -> Arg RawOpts
@@ -166,7 +178,7 @@ defMode =   Mode {
  ,modeGroupFlags = Group {
      groupNamed = []
     ,groupUnnamed = [
-        flagNone ["help","h","?"] (setboolopt "help") "Show command help."
+        flagNone ["h"] (setboolopt "h") "Show command usage."
         ]
     ,groupHidden = []
     }
@@ -222,11 +234,22 @@ standardAddonsHelp = [
   ,("addon9", "dummy add-on command for testing")
   ]
 
--- | Get a mode's help message as a nicely wrapped string.
-showModeHelp :: Mode a -> String
-showModeHelp = (showText defaultWrap :: [Text] -> String) .
+-- | Get a mode's usage message as a nicely wrapped string.
+showModeUsage :: Mode a -> String
+showModeUsage = (showText defaultWrap :: [Text] -> String) .
                (helpText [] HelpFormatDefault :: Mode a -> [Text])
 
+-- | Get the most appropriate documentation topic for a mode.
+-- Currently, that is either the hledger, hledger-ui, hledger-web or
+-- hledger-api manual.
+topicForMode :: Mode a -> Topic
+topicForMode m
+  | n == "hledger-ui"  = "ui"
+  | n == "hledger-web" = "web"
+  --  | n == "hledger-api" = lookupDocTxt "api" -- hledger-api uses docopt
+  | otherwise          = "cli"
+  where n = headDef "" $ modeNames m
+
 -- | Add command aliases to the command's help string.
 withAliases :: String -> [String] -> String
 s `withAliases` []     = s
@@ -248,7 +271,7 @@ s `withAliases` as = s ++ " (" ++ intercalate ", " as ++ ")"
 
 -- | Command line options, used in the @hledger@ package and above.
 -- This is the \"opts\" used throughout hledger CLI code.
--- representing the options that arguments that were provided at
+-- representing the options and arguments that were provided at
 -- startup on the command-line.
 data CliOpts = CliOpts {
      rawopts_         :: RawOpts
@@ -314,11 +337,11 @@ rawOptsToCliOpts rawopts = checkCliOpts <$> do
   return defcliopts {
               rawopts_         = rawopts
              ,command_         = stringopt "command" rawopts
-             ,file_            = map stripquotes $ listofstringopt "file" rawopts
+             ,file_            = map (T.unpack . stripquotes . T.pack) $ listofstringopt "file" rawopts
              ,rules_file_      = maybestringopt "rules-file" rawopts
              ,output_file_     = maybestringopt "output-file" rawopts
              ,output_format_   = maybestringopt "output-format" rawopts
-             ,alias_           = map stripquotes $ listofstringopt "alias" rawopts
+             ,alias_           = map (T.unpack . stripquotes . T.pack) $ listofstringopt "alias" rawopts
              ,debug_           = intopt "debug" rawopts
              ,ignore_assertions_ = boolopt "ignore-assertions" rawopts
              ,no_new_accounts_ = boolopt "no-new-accounts" rawopts -- add
@@ -348,8 +371,8 @@ getCliOpts mode' = do
   opts <- rawOptsToCliOpts rawopts
   debugArgs args' opts
   -- if any (`elem` args) ["--help","-h","-?"]
-  when ("help" `inRawOpts` rawopts_ opts) $
-    putStr (showModeHelp mode') >> exitSuccess
+  when ("h" `inRawOpts` rawopts_ opts) $ putStr (showModeUsage mode') >> exitSuccess
+  when ("help" `inRawOpts` rawopts_ opts) $ printHelpForTopic (topicForMode mode') >> exitSuccess
   return opts
   where
     -- | Print debug info about arguments and options if --debug is present.
@@ -367,7 +390,7 @@ getCliOpts mode' = do
 
 -- | Get the account name aliases from options, if any.
 aliasesFromOpts :: CliOpts -> [AccountAlias]
-aliasesFromOpts = map (\a -> fromparse $ runParser accountaliasp () ("--alias "++quoteIfNeeded a) a)
+aliasesFromOpts = map (\a -> fromparse $ runParser accountaliasp ("--alias "++quoteIfNeeded a) $ T.pack a)
                   . alias_
 
 -- | Get the (tilde-expanded, absolute) journal file path from
@@ -433,7 +456,7 @@ rulesFilePathFromOpts opts = do
 widthFromOpts :: CliOpts -> Int
 widthFromOpts CliOpts{width_=Nothing, available_width_=w} = w
 widthFromOpts CliOpts{width_=Just s}  =
-    case runParser (read `fmap` many1 digit <* eof) () "(unknown)" s of
+    case runParser (read `fmap` some digitChar <* eof :: ParsecT Dec String Identity Int) "(unknown)" s of
         Left e   -> optserror $ "could not parse width option: "++show e
         Right w  -> w
 
@@ -451,14 +474,14 @@ widthFromOpts CliOpts{width_=Just s}  =
 registerWidthsFromOpts :: CliOpts -> (Int, Maybe Int)
 registerWidthsFromOpts CliOpts{width_=Nothing, available_width_=w} = (w, Nothing)
 registerWidthsFromOpts CliOpts{width_=Just s}  =
-    case runParser registerwidthp () "(unknown)" s of
+    case runParser registerwidthp "(unknown)" s of
         Left e   -> optserror $ "could not parse width option: "++show e
         Right ws -> ws
     where
-        registerwidthp :: Stream [Char] m t => ParsecT [Char] st m (Int, Maybe Int)
+        registerwidthp :: (Stream s, Char ~ Token s) => ParsecT Dec s m (Int, Maybe Int)
         registerwidthp = do
-          totalwidth <- read `fmap` many1 digit
-          descwidth <- optionMaybe (char ',' >> read `fmap` many1 digit)
+          totalwidth <- read `fmap` some digitChar
+          descwidth <- optional (char ',' >> read `fmap` some digitChar)
           eof
           return (totalwidth, descwidth)
 
@@ -536,12 +559,12 @@ hledgerExecutablesInPath = do
 -- isExecutable f = getPermissions f >>= (return . executable)
 
 isHledgerExeName :: String -> Bool
-isHledgerExeName = isRight . parsewith hledgerexenamep
+isHledgerExeName = isRight . parsewith hledgerexenamep . T.pack
     where
       hledgerexenamep = do
         _ <- string progname
         _ <- char '-'
-        _ <- many1 (noneOf ".")
+        _ <- some (noneOf ".")
         optional (string "." >> choice' (map string addonExtensions))
         eof
 
