@@ -1,10 +1,11 @@
-{-# LANGUAGE CPP #-}
 {-|
 
 A ledger-compatible @register@ command.
 
 -}
 
+{-# LANGUAGE CPP, OverloadedStrings #-}
+
 module Hledger.Cli.Register (
   registermode
  ,register
@@ -16,6 +17,8 @@ module Hledger.Cli.Register (
 
 import Data.List
 import Data.Maybe
+-- import Data.Text (Text)
+import qualified Data.Text as T
 import System.Console.CmdArgs.Explicit
 import Text.CSV
 import Test.HUnit
@@ -29,19 +32,22 @@ registermode = (defCommandMode $ ["register"] ++ aliases) {
   modeHelp = "show postings and running total" `withAliases` aliases
  ,modeGroupFlags = Group {
      groupUnnamed = [
-      flagNone ["historical","H"] (\opts -> setboolopt "historical" opts) "include prior postings in the running total"
-     ,flagNone ["average","A"] (\opts -> setboolopt "average" opts) "show a running average instead of the running total (implies --empty)"
+      flagNone ["cumulative"]         (\opts -> setboolopt "change" opts)
+        "show running total from report start date (default)"
+     ,flagNone ["historical","H"] (\opts -> setboolopt "historical" opts)
+        "show historical running total/balance (includes postings before report start date)\n "
+     ,flagNone ["average","A"] (\opts -> setboolopt "average" opts)
+        "show running average of posting amounts instead of total (implies --empty)"
      ,flagNone ["related","r"] (\opts -> setboolopt "related" opts) "show postings' siblings instead"
      ,flagReq  ["width","w"] (\s opts -> Right $ setopt "width" s opts) "N"
-      (unlines
-       ["set output width (default:"
+      ("set output width (default: " ++
 #ifdef mingw32_HOST_OS
-       ,(show defaultWidth)
+       show defaultWidth
 #else
-       ,"terminal width"
+       "terminal width"
 #endif
-       ,"or COLUMNS. -wN,M sets description width as well)"
-       ])
+       ++ " or $COLUMNS). -wN,M sets description width as well."
+       )
     ]
      ++ outputflags
     ,groupHidden = []
@@ -61,16 +67,17 @@ register opts@CliOpts{reportopts_=ropts} j = do
 
 postingsReportAsCsv :: PostingsReport -> CSV
 postingsReportAsCsv (_,is) =
-  ["date","description","account","amount","running total or balance"]
+  ["txnidx","date","description","account","amount","total"]
   :
   map postingsReportItemAsCsvRecord is
 
 postingsReportItemAsCsvRecord :: PostingsReportItem -> Record
-postingsReportItemAsCsvRecord (_, _, _, p, b) = [date,desc,acct,amt,bal]
+postingsReportItemAsCsvRecord (_, _, _, p, b) = [idx,date,desc,acct,amt,bal]
   where
-    date = showDate $ postingDate p
-    desc = maybe "" tdescription $ ptransaction p
-    acct = bracket $ paccount p
+    idx  = show $ maybe 0 tindex $ ptransaction p
+    date = showDate $ postingDate p -- XXX csv should show date2 with --date2
+    desc = T.unpack $ maybe "" tdescription $ ptransaction p
+    acct = bracket $ T.unpack $ paccount p
       where
         bracket = case ptype p of
                              BalancedVirtualPosting -> (\s -> "["++s++"]")
@@ -108,7 +115,7 @@ tests_postingsReportAsText = [
 -- If description's width is specified, account will use the remaining space.
 -- Otherwise, description and account divide up the space equally.
 --
--- With a reporting interval, the layout is like so:
+-- With a report interval, the layout is like so:
 -- @
 -- <---------------- width (specified, terminal width, or 80) -------------------->
 -- date (21)              account                        amount (12)   balance (12)
@@ -173,7 +180,7 @@ postingsReportItemAsText opts preferredamtwidth preferredbalwidth (mdate, mendda
 
       -- gather content
       desc = fromMaybe "" mdesc
-      acct = parenthesise $ elideAccountName awidth $ paccount p
+      acct = parenthesise $ T.unpack $ elideAccountName awidth $ paccount p
          where
           (parenthesise, awidth) =
             case ptype p of
