@@ -23,8 +23,15 @@ module Hledger.Cli.Utils
     )
 where
 import Control.Exception as C
+import Data.Hashable (hash)
 import Data.List
 import Data.Maybe
+import Data.Text (Text)
+import qualified Data.Text as T
+import qualified Data.Text.IO as T
+import Data.Time (Day)
+import Data.Word
+import Numeric
 import Safe (readMay)
 import System.Console.CmdArgs
 import System.Directory (getModificationTime, getDirectoryContents, copyFile)
@@ -59,17 +66,58 @@ import Hledger.Read
 import Hledger.Utils
 
 
--- | Parse the user's specified journal file and run a hledger command on
--- it, or throw an error.
+-- | Parse the user's specified journal file, maybe apply some transformations
+-- (aliases, pivot) and run a hledger command on it, or throw an error.
 withJournalDo :: CliOpts -> (CliOpts -> Journal -> IO ()) -> IO ()
 withJournalDo opts cmd = do
   -- We kludgily read the file before parsing to grab the full text, unless
   -- it's stdin, or it doesn't exist and we are adding. We read it strictly
   -- to let the add command work.
   rulespath <- rulesFilePathFromOpts opts
-  journalpath <- journalFilePathFromOpts opts
-  ej <- readJournalFiles Nothing rulespath (not $ ignore_assertions_ opts) journalpath
-  either error' (cmd opts . journalApplyAliases (aliasesFromOpts opts)) ej
+  journalpaths <- journalFilePathFromOpts opts
+  ej <- readJournalFiles Nothing rulespath (not $ ignore_assertions_ opts) journalpaths
+  either error' (cmd opts . pivotByOpts opts . anonymiseByOpts opts . journalApplyAliases (aliasesFromOpts opts)) ej
+
+-- | Apply the pivot transformation on a journal, if option is present.
+pivotByOpts :: CliOpts -> Journal -> Journal
+pivotByOpts opts =
+  case maybestringopt "pivot" . rawopts_ $ opts of
+    Just tag -> pivot $ T.pack tag
+    Nothing  -> id
+
+-- | Apply the pivot transformation by given tag on a journal.
+pivot :: Text -> Journal -> Journal
+pivot tag j = j{jtxns = map pivotTrans . jtxns $ j}
+ where
+  pivotTrans t = t{tpostings = map pivotPosting . tpostings $ t}
+  pivotPosting p
+    | Just (_ , value) <- tagTuple = p{paccount = joinAccountNames tag value}
+    | _                <- tagTuple = p
+   where tagTuple = find ((tag ==) . fst) . ptags $ p
+
+-- | Apply the anonymisation transformation on a journal, if option is present
+anonymiseByOpts :: CliOpts -> Journal -> Journal
+anonymiseByOpts opts =
+  case maybestringopt "anon" . rawopts_ $ opts of
+    Just _  -> anonymise
+    Nothing -> id
+
+-- | Apply the anonymisation transformation on a journal
+anonymise :: Journal -> Journal
+anonymise j
+  = let
+      pAnons p = p { paccount = T.intercalate (T.pack ":") . map anon . T.splitOn (T.pack ":") . paccount $ p
+                   , pcomment = T.empty
+                   , ptransaction = fmap tAnons . ptransaction $ p
+                   }
+      tAnons txn = txn { tpostings = map pAnons . tpostings $ txn
+                       , tdescription = anon . tdescription $ txn
+                       , tcomment = T.empty
+                       }
+    in
+      j { jtxns = map tAnons . jtxns $ j }
+  where
+    anon = T.pack . flip showHex "" . (fromIntegral :: Int -> Word32) . hash
 
 -- | Write some output to stdout or to a file selected by --output-file.
 writeOutput :: CliOpts -> String -> IO ()
@@ -81,39 +129,46 @@ writeOutput opts s = do
 -- readJournalWithOpts :: CliOpts -> String -> IO Journal
 -- readJournalWithOpts opts s = readJournal Nothing Nothing Nothing s >>= either error' return
 
--- | Re-read a journal from its data file, or return an error string.
-journalReload :: Journal -> IO (Either String Journal)
-journalReload j = readJournalFile Nothing Nothing True $ journalFilePath j
+-- | Re-read the journal file(s) specified by options and maybe apply some
+-- transformations (aliases, pivot), or return an error string.
+-- Reads the full journal, without filtering.
+journalReload :: CliOpts -> IO (Either String Journal)
+journalReload opts = do
+  rulespath <- rulesFilePathFromOpts opts
+  journalpaths <- journalFilePathFromOpts opts
+  ((pivotByOpts opts . journalApplyAliases (aliasesFromOpts opts)) <$>) <$>
+    readJournalFiles Nothing rulespath (not $ ignore_assertions_ opts) journalpaths
 
--- | Re-read a journal from its data file mostly, only if the file has
--- changed since last read (or if there is no file, ie data read from
--- stdin). The provided options are mostly ignored. Return a journal or
--- the error message while reading it, and a flag indicating whether it
--- was re-read or not.
-journalReloadIfChanged :: CliOpts -> Journal -> IO (Either String Journal, Bool)
-journalReloadIfChanged _ j = do
+-- | Re-read the option-specified journal file(s), but only if any of
+-- them has changed since last read. (If the file is standard input,
+-- this will either do nothing or give an error, not tested yet).
+-- Returns a journal or error message, and a flag indicating whether
+-- it was re-read or not.  Like withJournalDo and journalReload, reads
+-- the full journal, without filtering.
+journalReloadIfChanged :: CliOpts -> Day -> Journal -> IO (Either String Journal, Bool)
+journalReloadIfChanged opts _d j = do
   let maybeChangedFilename f = do newer <- journalSpecifiedFileIsNewer j f
                                   return $ if newer then Just f else Nothing
   changedfiles <- catMaybes `fmap` mapM maybeChangedFilename (journalFilePaths j)
   if not $ null changedfiles
    then do
      whenLoud $ printf "%s has changed, reloading\n" (head changedfiles)
-     jE <- journalReload j
-     return (jE, True)
+     ej <- journalReload opts
+     return (ej, True)
    else
      return (Right j, False)
 
 -- | Has the journal's main data file changed since the journal was last
 -- read ?
 journalFileIsNewer :: Journal -> IO Bool
-journalFileIsNewer j@Journal{filereadtime=tread} = do
+journalFileIsNewer j@Journal{jlastreadtime=tread} = do
   tmod <- fileModificationTime $ journalFilePath j
   return $ diffClockTimes tmod tread > (TimeDiff 0 0 0 0 0 0 0)
 
 -- | Has the specified file (presumably one of journal's data files)
 -- changed since journal was last read ?
 journalSpecifiedFileIsNewer :: Journal -> FilePath -> IO Bool
-journalSpecifiedFileIsNewer Journal{filereadtime=tread} f = do
+journalSpecifiedFileIsNewer Journal{jlastreadtime=tread} f = do
   tmod <- fileModificationTime f
   return $ diffClockTimes tmod tread > (TimeDiff 0 0 0 0 0 0 0)
 
@@ -159,19 +214,19 @@ openBrowserOn u = trybrowsers browsers u
 -- overwrite it with this new text, or give an error, but only if the text
 -- is different from the current file contents, and return a flag
 -- indicating whether we did anything.
-writeFileWithBackupIfChanged :: FilePath -> String -> IO Bool
+writeFileWithBackupIfChanged :: FilePath -> T.Text -> IO Bool
 writeFileWithBackupIfChanged f t = do
   s <- readFile' f
   if t == s then return False
-            else backUpFile f >> writeFile f t >> return True
+            else backUpFile f >> T.writeFile f t >> return True
 
 -- | Back up this file with a (incrementing) numbered suffix, then
 -- overwrite it with this new text, or give an error.
 writeFileWithBackup :: FilePath -> String -> IO ()
 writeFileWithBackup f t = backUpFile f >> writeFile f t
 
-readFileStrictly :: FilePath -> IO String
-readFileStrictly f = readFile' f >>= \s -> C.evaluate (length s) >> return s
+readFileStrictly :: FilePath -> IO T.Text
+readFileStrictly f = readFile' f >>= \s -> C.evaluate (T.length s) >> return s
 
 -- | Back up this file with a (incrementing) numbered suffix, or give an error.
 backUpFile :: FilePath -> IO ()
