@@ -1,10 +1,10 @@
 -- bench
 -- By default, show approximate times for some standard hledger operations on a sample journal.
 -- With --criterion, show accurate times (slow).
--- With --simplebench, show approximate times for the commands in default.bench, using the first hledger executable on $PATH.
+-- TODO With --quickbench, show approximate times for the commands in default.bench, using the first hledger executable on $PATH.
 
 import Criterion.Main     (defaultMainWith, defaultConfig, bench, nfIO)
-import SimpleBench        (defaultMain)
+-- import QuickBench        (defaultMain)
 import System.Directory   (getCurrentDirectory)
 import System.Environment (getArgs, withArgs)
 import System.Info        (os)
@@ -20,14 +20,18 @@ outputfile = "/dev/null" -- hide output of benchmarked commands (XXX unixism)
 -- outputfile = "-" -- show output of benchmarked commands
 
 main = do
- -- withArgs ["--simplebench"] $ do
+ -- withArgs ["--quickbench"] $ do
  -- withArgs ["--criterion"] $ do
   args <- getArgs
   if "--criterion" `elem` args
-    then withArgs [] benchWithCriterion
-    else if "--simplebench" `elem` args
-         then benchWithSimplebench
-         else benchWithTimeit
+  then
+    withArgs [] benchWithCriterion
+  else
+--     if "--quickbench" `elem` args
+--     then
+--       benchWithQuickbench
+--     else
+      benchWithTimeit
 
 benchWithTimeit = do
   getCurrentDirectory >>= printf "Benchmarking hledger in %s with timeit\n"
@@ -36,8 +40,7 @@ benchWithTimeit = do
   (t1,_) <- timeit ("print") $ print' opts j
   (t2,_) <- timeit ("register") $ register opts j
   (t3,_) <- timeit ("balance") $ balance  opts j
-  (t4,_) <- timeit ("stats") $ stats opts j
-  printf "Total: %0.2fs\n" (sum [t0,t1,t2,t3,t4])
+  printf "Total: %0.2fs\n" (sum [t0,t1,t2,t3])
 
 timeit :: String -> IO a -> IO (Double, a)
 timeit name action = do
@@ -58,13 +61,13 @@ benchWithCriterion = do
     bench ("stats")            $ nfIO $ stats    opts j
     ]
 
-benchWithSimplebench = do
-  let whichcmd = if os == "mingw32" then "where" else "which"
-  exe <- init <$> readProcess whichcmd ["hledger"] ""
-  pwd <- getCurrentDirectory
-  printf "Benchmarking %s in %s with simplebench and shell\n" exe pwd
-  flip withArgs SimpleBench.defaultMain [
-     "-fbench/default.bench"
-    ,"-v"
-    ,"hledger"
-    ] 
+-- benchWithQuickbench = do
+--   let whichcmd = if os == "mingw32" then "where" else "which"
+--   exe <- init <$> readProcess whichcmd ["hledger"] ""
+--   pwd <- getCurrentDirectory
+--   printf "Benchmarking %s in %s with quickbench and shell\n" exe pwd
+--   flip withArgs QuickBench.defaultMain [
+--      "-fbench/default.bench"
+--     ,"-v"
+--     ,"hledger"
+--     ]
